{*************************************************************}
{              KDatabase Export pack version 1.00             }
{               Copyright  2000 Korzh company                }
{                http://www.korzh.com/delphi/                 }
{                  mailto:delphi@korzh.com                    }
{-------------------------------------------------------------}
{                   TKDatasetToPrinter component                  }
{                  last updated: Mar-16-2000                  }
{*************************************************************}
unit kds2prn;

interface
uses Windows, Messages, Classes, Dialogs, SysUtils, Graphics, Forms, DB, DBTables;

{$I defines.inc}

type
  TKDBExportError = class(Exception)
  end;

  TKWorkKind = (wkIdle, wkPrinting, wkPreview);

  TKDatasetToPrinter = class(TComponent)
  private
    FDataSet : TDataSet;

    FTitle : string;
    FFooter : TStrings;
    FHeader : TStrings;
    FLineHeight : integer;
    FExtLineHeight : integer;
    FZoom : extended;      //Zoom factor; Reserved
    FTotalWidth : integer; //Total width in pixels of all printed fields
    FPrintedFields : TStringList;
    FWorkKind : TKWorkKind;
    FPageWidth : integer;
    FPageHeight : integer;
    FLeftMargin : integer;
    FTopMargin : integer;
    FCanvas : TCanvas;  //canvas of the device to print
    FScaleX : integer;
    FScaleY : integer;
    FTopHeight : integer;
    FFooterHeight : integer;
    FTitleFont : TFont;
    FFont : TFont;

    procedure SetFooter(Value : TStrings);
    procedure SetHeader(Value : TStrings);
    function GetWorkAborted :boolean;
    procedure InitOutput;
    procedure StartNewPage;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);override;
    procedure DrawResult;virtual;
  public
    constructor Create(AOwner : TComponent);override;
    destructor Destroy; override;
    procedure Preview;
    procedure Print;
    property WorkAborted : boolean read GetWorkAborted;
  published
    property DataSet : TDataSet read FDataSet write FDataSet;
    property Footer : TStrings read FFooter write SetFooter;
    property Header : TStrings read FHeader write SetHeader;
    property LeftMargin : integer read FLeftMargin write FLeftMargin default 10;
    property TopMargin : integer read FTopMargin write FTopMargin default 5;
    property Title : string read FTitle write FTitle;
  end;

  //not realised now.
  TKPreviewForm = class(TCustomForm)
  private
    FDSReporter : TKDatasetToPrinter;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
  public
{$IFDEF D4_PLUS}
    constructor CreateNew(AOwner : TComponent; Dummy : integer = 0); override;
{$ELSE}
    constructor CreateNew(AOwner : TComponent);
{$ENDIF}
    property DSReporter : TKDatasetToPrinter read FDSReporter write FDSReporter;
  end;

implementation
uses printers;

function ReplSymb(Symbol : char; Count : integer) : string;
var
  i : integer;
begin
  Result := '';
  if Count < 0 then exit;
  for i := 1 to Count do
    Result := Result + Symbol;
end;

constructor TKDatasetToPrinter.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FDataset := nil;
  FHeader := TStringList.Create;
  FFooter := TStringList.Create;
  FPrintedFields := TStringList.Create;
  FWorkKind := wkIdle;
  FLeftMargin := 10;
  FTopMargin := 5;
  
  FFont := TFont.Create;
  FFont.Name := 'Arial';
  FFont.Size := 10;
  FFont.Style := [];
  FTitleFont := TFont.Create;
  FTitleFont.Name := 'Arial';
  FTitleFont.Size := 16;
  FTitleFont.Style := [fsBold];
end;

destructor TKDatasetToPrinter.Destroy;
begin
  FFont.Free;
  FTitleFont.Free;
  FPrintedFields.Free;
  FHeader.Free;
  FFooter.Free;
  inherited;
end;

procedure TKDatasetToPrinter.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDataSet) then
    FDataSet := nil;
end;

procedure TKDatasetToPrinter.SetFooter(Value : TStrings);
begin
  FHeader.Assign(Value);
end;

procedure TKDatasetToPrinter.SetHeader(Value : TStrings);
begin
  FFooter.Assign(Value);
end;

function TKDatasetToPrinter.GetWorkAborted :boolean;
begin
  if FWorkKind = wkPrinting then
    Result := Printer.Aborted
  else
    Result := false;
end;

procedure TKDatasetToPrinter.StartNewPage;
begin
  if FWorkKind = wkPrinting then
    Printer.NewPage;
end;

procedure TKDatasetToPrinter.InitOutput;
var
  i, FldWidth : integer;
begin
  FScaleX := trunc(GetDeviceCaps(FCanvas.Handle, LOGPIXELSX) / 254 * 10);
  FScaleY := trunc(GetDeviceCaps(FCanvas.Handle, LOGPIXELSY) / 254 * 10);

  FTotalWidth := 0;
  FPrintedFields.Clear;
  for i := 0 to FDataset.FieldCount - 1 do
  begin
    if FDataset.Fields[i].Visible and
       not (FDataSet.Fields[i].DataType in [ftBlob, ftMemo, ftGraphic, ftFmtMemo,
                                            ftParadoxOle, ftDBaseOle, ftTypedBinary])
    then begin
      FldWidth := FCanvas.TextWidth(ReplSymb('e', FDataSet.Fields[i].DisplayWidth) + ' ');
      inc(FTotalWidth, FldWidth);
      FPrintedFields.AddObject(FDataset.Fields[i].FieldName, Pointer(FldWidth));
    end;
  end;

  FLineHeight := FCanvas.TextHeight('X') + FScaleY; //height of text + 1mm
  FExtLineHeight := trunc(FLineHeight * 1.5);
  FTopHeight := FLineHeight * FHeader.Count + FTopMargin * FScaleY;
  if FTitle <> '' then inc(FTopHeight, FLineHeight);
  FFooterHeight := FLineHeight * FFooter.Count;
end;

procedure TKDatasetToPrinter.DrawResult;
var
  X, Y : integer;

  procedure PrintColumnNames;
  var
    i: integer;
    Fld : TField;
    FldWidth : integer;
  begin
    //Printing columns
    X := FLeftMargin * FScaleX;
    for i := 0 to FPrintedFields.Count - 1 do
    begin
      Fld := FDataset.FindField(FPrintedFields[i]);
      FldWidth := integer(FPrintedFields.Objects[i]);
      FCanvas.TextOut(X, Y, Fld.DisplayLabel);
      inc(X, FldWidth);
    end;
    inc(Y, FLineHeight);
  end;

  procedure PrintRow;
  var
    i: integer;
    Fld : TField;
    FldWidth : integer;
  begin
    X := FLeftMargin * FScaleX;
    for i := 0 to FPrintedFields.Count - 1 do
    begin
      Fld := FDataset.FindField(FPrintedFields[i]);
      FldWidth := integer(FPrintedFields.Objects[i]);
      FCanvas.TextOut(X, Y, Fld.AsString);
      inc(X, FldWidth);
    end;
    inc(Y, FLineHeight);
  end;

  procedure PrintHeaderFooter(Strings : TStrings);
  var
    i : integer;
  begin
    for i := 0 to Strings.Count - 1 do
    begin
      X := (FPageWidth - FCanvas.TextWidth(Strings[i])) div 2;
      FCanvas.TextOut(X, Y, Strings[i]);
      inc(Y, FLineHeight);
    end;
  end;

  procedure PrintTop;
  begin
    X := FLeftMargin * FScaleX;
    Y := FTopMargin * FScaleY;
    FCanvas.Font := FFont;
    PrintHeaderFooter(FHeader);
    inc(Y, 2 * FLineHeight);

    //printing title
    if FTitle <> '' then
    begin
      FCanvas.Font := FTitleFont;
      X := (FPageWidth - FCanvas.TextWidth(FTitle)) div 2;
      FCanvas.TextOut(X, Y, FTitle);
      inc(Y, 2 * FLineHeight);
    end;

    FCanvas.Font := FFont;
    FCanvas.Font.Style := [fsBold];
    PrintColumnNames;
    FCanvas.Font.Style := [];
  end;

begin
  PrintTop;
  FDataset.First;
  while not FDataset.EOF do
  begin
    PrintRow;
    FDataset.Next;
    if Y + FLineHeight > FPageHeight - FFooterHeight then
    begin
      if not WorkAborted then
      begin
        //printing footer
        Y := FPageHeight - FFooterHeight - 10;
        PrintHeaderFooter(FFooter);
        StartNewPage;
      end;
      PrintTop;
    end;
  end;
  //printing footer
  Y := FPageHeight - FFooterHeight - 10;
  PrintHeaderFooter(FFooter);
end;

procedure TKDatasetToPrinter.Print;
begin
  if FDataSet = nil then
    raise TKDBExportError.Create('Dataset is not specified');
  if not FDataSet.Active then
    raise TKDBExportError.Create('Dataset is not active');
  Printer.BeginDoc;
  FWorkKind := wkPrinting;
  FCanvas := Printer.Canvas;
//  if SetMapMode(FCanvas.Handle, MM_LOMETRIC) = 0 then ShowMessage('Fail');
  FPageHeight := Printer.PageHeight;
  FPageWidth  := Printer.PageWidth;
  InitOutput;
//  FScale := 1; //Printer.PageWidth div FTotalWidth;
  DrawResult;
  Printer.EndDoc;
end;

procedure TKDatasetToPrinter.Preview;
var
  PreviewForm : TKPreviewForm;
begin
  if FDataSet = nil then
    raise TKDBExportError.Create('Dataset is not specified');
  if not FDataSet.Active then
    raise TKDBExportError.Create('Dataset is not active');

  FWorkKind := wkPreview;

  PreviewForm := TKPreviewForm.CreateNew(nil);
  with PreviewForm do
  begin
    try
      Left := 10;
      Width := Screen.Width - 20;
      Top := 0;
      Height := Screen.Height - 60;
      BorderStyle := bsToolWindow;
      Position := poScreenCenter;
      Color := clWindow;
      DSReporter := Self;
      FCanvas := Canvas;
      FPageWidth := ClientWidth;
      FPageHeight := ClientHeight;
      InitOutput;
//      FScale := 1;
      ShowModal;
    finally
      Free;
    end;
  end;
end;

{$IFDEF D4_PLUS}
constructor TKPreviewForm.CreateNew(AOwner : TComponent; Dummy : integer = 0); 
{$ELSE}
constructor TKPreviewForm.CreateNew(AOwner : TComponent);
{$ENDIF}
begin
  inherited CreateNew(AOwner);
  FDSReporter := nil;
end;

procedure TKPreviewForm.WMPaint(var Message: TWMPaint);
begin
  inherited;
  if FDSReporter <> nil then
    FDSReporter.DrawResult;
end;


end.
